# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from datagrid_ai import Datagrid, AsyncDatagrid
from tests.utils import assert_matches_type
from datagrid_ai.pagination import SyncCursorIDPage, AsyncCursorIDPage
from datagrid_ai.types.organization import (
    Teamspace,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTeamspaces:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Datagrid) -> None:
        teamspace = client.organization.teamspaces.create(
            access="open",
            name="name",
        )
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Datagrid) -> None:
        teamspace = client.organization.teamspaces.create(
            access="open",
            name="name",
            cloud_provider="aws",
        )
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Datagrid) -> None:
        response = client.organization.teamspaces.with_raw_response.create(
            access="open",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        teamspace = response.parse()
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Datagrid) -> None:
        with client.organization.teamspaces.with_streaming_response.create(
            access="open",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            teamspace = response.parse()
            assert_matches_type(Teamspace, teamspace, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Datagrid) -> None:
        teamspace = client.organization.teamspaces.retrieve(
            "teamspace_id",
        )
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Datagrid) -> None:
        response = client.organization.teamspaces.with_raw_response.retrieve(
            "teamspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        teamspace = response.parse()
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Datagrid) -> None:
        with client.organization.teamspaces.with_streaming_response.retrieve(
            "teamspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            teamspace = response.parse()
            assert_matches_type(Teamspace, teamspace, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Datagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `teamspace_id` but received ''"):
            client.organization.teamspaces.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: Datagrid) -> None:
        teamspace = client.organization.teamspaces.list()
        assert_matches_type(SyncCursorIDPage[Teamspace], teamspace, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Datagrid) -> None:
        teamspace = client.organization.teamspaces.list(
            after="after",
            before="before",
            limit=1,
        )
        assert_matches_type(SyncCursorIDPage[Teamspace], teamspace, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Datagrid) -> None:
        response = client.organization.teamspaces.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        teamspace = response.parse()
        assert_matches_type(SyncCursorIDPage[Teamspace], teamspace, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Datagrid) -> None:
        with client.organization.teamspaces.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            teamspace = response.parse()
            assert_matches_type(SyncCursorIDPage[Teamspace], teamspace, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_patch(self, client: Datagrid) -> None:
        teamspace = client.organization.teamspaces.patch(
            teamspace_id="teamspace_id",
        )
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    def test_method_patch_with_all_params(self, client: Datagrid) -> None:
        teamspace = client.organization.teamspaces.patch(
            teamspace_id="teamspace_id",
            access="open",
            name="name",
        )
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    def test_raw_response_patch(self, client: Datagrid) -> None:
        response = client.organization.teamspaces.with_raw_response.patch(
            teamspace_id="teamspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        teamspace = response.parse()
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    def test_streaming_response_patch(self, client: Datagrid) -> None:
        with client.organization.teamspaces.with_streaming_response.patch(
            teamspace_id="teamspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            teamspace = response.parse()
            assert_matches_type(Teamspace, teamspace, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_patch(self, client: Datagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `teamspace_id` but received ''"):
            client.organization.teamspaces.with_raw_response.patch(
                teamspace_id="",
            )


class TestAsyncTeamspaces:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncDatagrid) -> None:
        teamspace = await async_client.organization.teamspaces.create(
            access="open",
            name="name",
        )
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncDatagrid) -> None:
        teamspace = await async_client.organization.teamspaces.create(
            access="open",
            name="name",
            cloud_provider="aws",
        )
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.organization.teamspaces.with_raw_response.create(
            access="open",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        teamspace = await response.parse()
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncDatagrid) -> None:
        async with async_client.organization.teamspaces.with_streaming_response.create(
            access="open",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            teamspace = await response.parse()
            assert_matches_type(Teamspace, teamspace, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncDatagrid) -> None:
        teamspace = await async_client.organization.teamspaces.retrieve(
            "teamspace_id",
        )
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.organization.teamspaces.with_raw_response.retrieve(
            "teamspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        teamspace = await response.parse()
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncDatagrid) -> None:
        async with async_client.organization.teamspaces.with_streaming_response.retrieve(
            "teamspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            teamspace = await response.parse()
            assert_matches_type(Teamspace, teamspace, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncDatagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `teamspace_id` but received ''"):
            await async_client.organization.teamspaces.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncDatagrid) -> None:
        teamspace = await async_client.organization.teamspaces.list()
        assert_matches_type(AsyncCursorIDPage[Teamspace], teamspace, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncDatagrid) -> None:
        teamspace = await async_client.organization.teamspaces.list(
            after="after",
            before="before",
            limit=1,
        )
        assert_matches_type(AsyncCursorIDPage[Teamspace], teamspace, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.organization.teamspaces.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        teamspace = await response.parse()
        assert_matches_type(AsyncCursorIDPage[Teamspace], teamspace, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncDatagrid) -> None:
        async with async_client.organization.teamspaces.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            teamspace = await response.parse()
            assert_matches_type(AsyncCursorIDPage[Teamspace], teamspace, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_patch(self, async_client: AsyncDatagrid) -> None:
        teamspace = await async_client.organization.teamspaces.patch(
            teamspace_id="teamspace_id",
        )
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    async def test_method_patch_with_all_params(self, async_client: AsyncDatagrid) -> None:
        teamspace = await async_client.organization.teamspaces.patch(
            teamspace_id="teamspace_id",
            access="open",
            name="name",
        )
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    async def test_raw_response_patch(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.organization.teamspaces.with_raw_response.patch(
            teamspace_id="teamspace_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        teamspace = await response.parse()
        assert_matches_type(Teamspace, teamspace, path=["response"])

    @parametrize
    async def test_streaming_response_patch(self, async_client: AsyncDatagrid) -> None:
        async with async_client.organization.teamspaces.with_streaming_response.patch(
            teamspace_id="teamspace_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            teamspace = await response.parse()
            assert_matches_type(Teamspace, teamspace, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_patch(self, async_client: AsyncDatagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `teamspace_id` but received ''"):
            await async_client.organization.teamspaces.with_raw_response.patch(
                teamspace_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["ConversationCreateParams"]


class ConversationCreateParams(TypedDict, total=False):
    name: Optional[str]
    """Name for the conversation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ConversationSortField"]

ConversationSortField: TypeAlias = Literal["created_at", "updated_at"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import voice_start_session_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.voice_session_response import VoiceSessionResponse

__all__ = ["VoiceResource", "AsyncVoiceResource"]


class VoiceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VoiceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return VoiceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VoiceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return VoiceResourceWithStreamingResponse(self)

    def start_session(
        self,
        *,
        agent_id: Optional[str] | Omit = omit,
        config: Optional[voice_start_session_params.Config] | Omit = omit,
        conversation_id: Optional[str] | Omit = omit,
        ephemeral: Optional[bool] | Omit = omit,
        file_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        initial_context: Optional[str] | Omit = omit,
        knowledge_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        page_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        secret_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        user: Optional[voice_start_session_params.User] | Omit = omit,
        voice_config: Optional[voice_start_session_params.VoiceConfig] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceSessionResponse:
        """
        Prepare a real-time voice conversation with an AI Agent.

        Returns a WebSocket URL and a ready-made `start` message. Open a WebSocket
        connection to the returned `url`, send `start_message` as the first frame, then
        stream audio back and forth.

        You can also skip this endpoint and connect directly:
        `wss://api.datagrid.com/ws/voice?token=YOUR_API_KEY`

        **WebSocket Protocol:**

        Once connected, send a JSON message with `type: "start"` and the session
        parameters as the payload. The server responds with `type: "started"` containing
        the session and conversation IDs, followed by `type: "ready"` when the agent is
        ready to receive audio.

        **Audio Format:**

        - Client → Server: 16-bit mono PCM at 16kHz, base64-encoded
        - Server → Client: 16-bit mono PCM at 24kHz, base64-encoded

        **Message Types:**

        - Client: `start`, `audio`, `stop`, `interrupt`, `text`
        - Server: `started`, `ready`, `audio`, `tool_call`, `interrupted`, `error`,
          `transcript`, `citation`, `ended`

        Args:
          agent_id: The ID of the agent to use for the voice conversation. If not provided, the
              default agent is used.

          config: Override the agent config for this voice session. Only prompt overrides are
              supported — voice sessions always use Gemini Live, so LLM model, agent model,
              planning prompt, and tool settings are not applicable.

          conversation_id: The ID of an existing conversation to continue. If not provided, a new
              conversation will be created.

          ephemeral: When true, the session is ephemeral and will not save messages to conversation
              history.

          file_ids: Array of file IDs to attach to the voice conversation.

          initial_context: Optional context text for the voice session. When provided, the AI will start by
              briefly explaining this content before listening for user input.

          knowledge_ids: Array of knowledge IDs to make accessible to the agent.

          page_ids: Array of page IDs to make accessible to the agent. The page and all knowledge
              under it will be accessible.

          secret_ids: Array of secret IDs to include in the context.

          user: User information override for converse calls. All fields are optional - only
              provided fields will override the default user information.

          voice_config: Voice session configuration options.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/voice",
            body=maybe_transform(
                {
                    "agent_id": agent_id,
                    "config": config,
                    "conversation_id": conversation_id,
                    "ephemeral": ephemeral,
                    "file_ids": file_ids,
                    "initial_context": initial_context,
                    "knowledge_ids": knowledge_ids,
                    "page_ids": page_ids,
                    "secret_ids": secret_ids,
                    "user": user,
                    "voice_config": voice_config,
                },
                voice_start_session_params.VoiceStartSessionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceSessionResponse,
        )


class AsyncVoiceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVoiceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVoiceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVoiceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return AsyncVoiceResourceWithStreamingResponse(self)

    async def start_session(
        self,
        *,
        agent_id: Optional[str] | Omit = omit,
        config: Optional[voice_start_session_params.Config] | Omit = omit,
        conversation_id: Optional[str] | Omit = omit,
        ephemeral: Optional[bool] | Omit = omit,
        file_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        initial_context: Optional[str] | Omit = omit,
        knowledge_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        page_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        secret_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        user: Optional[voice_start_session_params.User] | Omit = omit,
        voice_config: Optional[voice_start_session_params.VoiceConfig] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceSessionResponse:
        """
        Prepare a real-time voice conversation with an AI Agent.

        Returns a WebSocket URL and a ready-made `start` message. Open a WebSocket
        connection to the returned `url`, send `start_message` as the first frame, then
        stream audio back and forth.

        You can also skip this endpoint and connect directly:
        `wss://api.datagrid.com/ws/voice?token=YOUR_API_KEY`

        **WebSocket Protocol:**

        Once connected, send a JSON message with `type: "start"` and the session
        parameters as the payload. The server responds with `type: "started"` containing
        the session and conversation IDs, followed by `type: "ready"` when the agent is
        ready to receive audio.

        **Audio Format:**

        - Client → Server: 16-bit mono PCM at 16kHz, base64-encoded
        - Server → Client: 16-bit mono PCM at 24kHz, base64-encoded

        **Message Types:**

        - Client: `start`, `audio`, `stop`, `interrupt`, `text`
        - Server: `started`, `ready`, `audio`, `tool_call`, `interrupted`, `error`,
          `transcript`, `citation`, `ended`

        Args:
          agent_id: The ID of the agent to use for the voice conversation. If not provided, the
              default agent is used.

          config: Override the agent config for this voice session. Only prompt overrides are
              supported — voice sessions always use Gemini Live, so LLM model, agent model,
              planning prompt, and tool settings are not applicable.

          conversation_id: The ID of an existing conversation to continue. If not provided, a new
              conversation will be created.

          ephemeral: When true, the session is ephemeral and will not save messages to conversation
              history.

          file_ids: Array of file IDs to attach to the voice conversation.

          initial_context: Optional context text for the voice session. When provided, the AI will start by
              briefly explaining this content before listening for user input.

          knowledge_ids: Array of knowledge IDs to make accessible to the agent.

          page_ids: Array of page IDs to make accessible to the agent. The page and all knowledge
              under it will be accessible.

          secret_ids: Array of secret IDs to include in the context.

          user: User information override for converse calls. All fields are optional - only
              provided fields will override the default user information.

          voice_config: Voice session configuration options.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/voice",
            body=await async_maybe_transform(
                {
                    "agent_id": agent_id,
                    "config": config,
                    "conversation_id": conversation_id,
                    "ephemeral": ephemeral,
                    "file_ids": file_ids,
                    "initial_context": initial_context,
                    "knowledge_ids": knowledge_ids,
                    "page_ids": page_ids,
                    "secret_ids": secret_ids,
                    "user": user,
                    "voice_config": voice_config,
                },
                voice_start_session_params.VoiceStartSessionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceSessionResponse,
        )


class VoiceResourceWithRawResponse:
    def __init__(self, voice: VoiceResource) -> None:
        self._voice = voice

        self.start_session = to_raw_response_wrapper(
            voice.start_session,
        )


class AsyncVoiceResourceWithRawResponse:
    def __init__(self, voice: AsyncVoiceResource) -> None:
        self._voice = voice

        self.start_session = async_to_raw_response_wrapper(
            voice.start_session,
        )


class VoiceResourceWithStreamingResponse:
    def __init__(self, voice: VoiceResource) -> None:
        self._voice = voice

        self.start_session = to_streamed_response_wrapper(
            voice.start_session,
        )


class AsyncVoiceResourceWithStreamingResponse:
    def __init__(self, voice: AsyncVoiceResource) -> None:
        self._voice = voice

        self.start_session = async_to_streamed_response_wrapper(
            voice.start_session,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .tool_name import ToolName
from .tool_param import ToolParam

__all__ = [
    "AgentUpdateParams",
    "Corpus",
    "CorpusCorpusKnowledgeItem",
    "CorpusCorpusPageItem",
    "DisabledTool",
    "McpServer",
    "Tool",
]


class AgentUpdateParams(TypedDict, total=False):
    agent_model: Union[Literal["magpie-1.1", "magpie-1.1-flash", "magpie-2.0", "magpie-2.5", "llm-only"], str, None]
    """The agent model determines the processing mode for Converse requests.

    Each model maps to one of three modes available in the Datagrid UI:

    **Agentic mode** (full tool use, planning, and multi-step reasoning):

    - `magpie-2.0` — Default. Agentic model with proactive planning and reasoning.
    - `magpie-2.5` — Beta. Our latest agentic model — faster, more adaptable, and
      built to handle a broader range of real-world tasks.
    - `magpie-1.1` — Previous-generation agentic model.

    **Ask mode** (lightweight, single-turn Q&A):

    - `magpie-1.1-flash` — Fast model optimized for RAG use cases. Only supports the
      `semantic_search` tool. A 400 error will be returned if other tools are
      specified. Structured outputs are not supported.

    **Fastest mode** (direct LLM response, no tool execution):

    - `llm-only` — Runs a direct LLM conversation with no planning or tool calls. A
      400 error will be returned if tools are specified. Structured outputs are not
      supported.

    Can also accept any custom string value for future model versions.
    """

    corpus: Optional[Iterable[Corpus]]
    """Array of corpus items the agent should use during the converse.

    When omitted, all knowledge is used.
    """

    custom_prompt: Optional[str]
    """Use custom prompt to instruct the style and formatting of the agent's response"""

    description: Optional[str]
    """The description of the agent"""

    disabled_tools: Optional[SequenceNotStr[DisabledTool]]
    """Array of the agent tools to disable.

    Disabling is performed after the 'agent_tools' rules are applied. For example,
    agent_tools: null and disabled_tools: [data_analysis] will enable everything but
    the data_analysis tool. If nothing or [] is provided, nothing is disabled and
    therefore only the agent_tools setting is relevant.
    """

    emoji: Optional[str]
    """The emoji of the agent"""

    knowledge_ids: Optional[SequenceNotStr[str]]
    """Deprecated, use corpus instead.

    Array of Knowledge IDs the agent should use during the converse. When omitted,
    all knowledge is used.
    """

    llm_model: Union[
        Literal[
            "gemini-3-pro-preview",
            "gemini-3.1-pro-preview",
            "gemini-3-flash-preview",
            "gemini-2.5-pro",
            "gemini-2.5-pro-preview-05-06",
            "gemini-2.5-flash",
            "gemini-2.5-flash-preview-04-17",
            "gemini-2.5-flash-native-audio-preview-12-2025",
            "gemini-2.5-flash-lite",
            "gpt-5",
            "gpt-5.1",
            "gemini-2.0-flash-001",
            "gemini-2.0-flash",
            "gemini-1.5-pro-001",
            "gemini-1.5-pro-002",
            "gemini-1.5-flash-002",
            "gemini-1.5-flash-001",
            "chatgpt-4o-latest",
            "gpt-4o",
            "gpt-4",
            "gpt-4-turbo",
            "gpt-4o-mini",
        ],
        str,
        None,
    ]
    """The LLM used to generate responses."""

    mcp_servers: Optional[Iterable[McpServer]]
    """Registered MCP servers to enable for this agent."""

    name: Optional[str]
    """The name of the agent"""

    planning_prompt: Optional[str]
    """
    Define the planning strategy your AI Agent should use when breaking down tasks
    and solving problems
    """

    system_prompt: Optional[str]
    """Directs your AI Agent's operational behavior."""

    tools: Optional[SequenceNotStr[Tool]]
    """Array of the agent tools to enable.

    If not provided, or null is provided - default tools of the agent are used. If
    empty list provided - none of the tools are used. When connection_id is set for
    a tool, it will use that specific connection instead of the default one.

    **Tool availability by agent model:**

    - **Agentic** (`magpie-2.0`, `magpie-2.5`, `magpie-1.1`): All tools below are
      available.
    - **Ask** (`magpie-1.1-flash`): Only `semantic_search` is supported. Requests
      specifying other tools will be rejected with a 400 error.
    - **Fastest** (`llm-only`): No tools are executed. Requests specifying tools
      will be rejected with a 400 error.

    Knowledge management tools:

    - data_analysis: Answer statistical or analytical questions like "Show my
      quarterly revenue growth"
    - semantic_search: Search knowledge through natural language queries.
    - agent_memory: Agents can remember experiences, conversations and user
      preferences.
    - schema_info: Helps the Agent understand column names and dataset purpose.
      Avoid disabling
    - table_info: Allow the AI Agent to get information about datasets and schemas
    - create_dataset: Agents respond with data tables

    Actions:

    - calendar: Allow the Agent to access and make changes to your Google Calendar
    - schedule_recurring_message_tool: Eliminate busywork such as: "Send a summary
      of today's meetings at 5pm on workdays"

    Data processing tools:

    - data_classification: Agents handle queries like "Label these emails as high,
      medium, or low priority"
    - data_extraction: Helps the agent understand data from other tools. Avoid
      disabling
    - image_detection: Extract information from images using AI
    - pdf_extraction: Extraction of information from PDFs using AI

    Enhanced response tools:

    - connect_data: Agents provide buttons to import data in response to queries
      like "Connect Hubspot"
    - download_data: Agents handle queries like "download the table as CSV"

    Web tools:

    - web_search: Agents search the internet, and provide links to their sources
    - fetch_url: Fetch URL content
    - company_prospect_researcher: Agents provide information about companies
    - people_prospect_researcher: Agents provide information about people
    """


class CorpusCorpusKnowledgeItem(TypedDict, total=False):
    knowledge_id: Required[str]
    """The ID of the knowledge to include in the corpus."""

    type: Required[Literal["knowledge"]]
    """The type of the corpus item. Always 'knowledge' for knowledge items."""


class CorpusCorpusPageItem(TypedDict, total=False):
    page_id: Required[str]
    """The ID of the page to include in the corpus."""

    type: Required[Literal["page"]]
    """The type of the corpus item. Always 'page' for page items."""


Corpus: TypeAlias = Union[CorpusCorpusKnowledgeItem, CorpusCorpusPageItem]

DisabledTool: TypeAlias = Union[ToolName, ToolParam]


class McpServer(TypedDict, total=False):
    server_id: Required[str]

    credential_id: Optional[str]


Tool: TypeAlias = Union[ToolName, ToolParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from .._models import BaseModel
from .identity_teamspace import IdentityTeamspace

__all__ = ["Identity"]


class Identity(BaseModel):
    current_teamspace_id: str
    """
    The teamspace ID this request is scoped to (derived from the API key or the
    Datagrid-Teamspace header).
    """

    object: Literal["identity"]

    teamspaces: List[IdentityTeamspace]
    """All teamspaces the authenticated user is a member of."""

    user_id: str
    """The ID of the authenticated user."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["KnowledgeConnectParams"]


class KnowledgeConnectParams(TypedDict, total=False):
    connection_id: Required[str]
    """The id of the connection to be used to create the knowledge."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["VoiceStartSessionParams", "Config", "User", "VoiceConfig"]


class VoiceStartSessionParams(TypedDict, total=False):
    agent_id: Optional[str]
    """The ID of the agent to use for the voice conversation.

    If not provided, the default agent is used.
    """

    config: Optional[Config]
    """
    Override the agent config for this voice session. Only prompt overrides are
    supported — voice sessions always use Gemini Live, so LLM model, agent model,
    planning prompt, and tool settings are not applicable.
    """

    conversation_id: Optional[str]
    """The ID of an existing conversation to continue.

    If not provided, a new conversation will be created.
    """

    ephemeral: Optional[bool]
    """
    When true, the session is ephemeral and will not save messages to conversation
    history.
    """

    file_ids: Optional[SequenceNotStr[str]]
    """Array of file IDs to attach to the voice conversation."""

    initial_context: Optional[str]
    """Optional context text for the voice session.

    When provided, the AI will start by briefly explaining this content before
    listening for user input.
    """

    knowledge_ids: Optional[SequenceNotStr[str]]
    """Array of knowledge IDs to make accessible to the agent."""

    page_ids: Optional[SequenceNotStr[str]]
    """Array of page IDs to make accessible to the agent.

    The page and all knowledge under it will be accessible.
    """

    secret_ids: Optional[SequenceNotStr[str]]
    """Array of secret IDs to include in the context."""

    user: Optional[User]
    """User information override for converse calls.

    All fields are optional - only provided fields will override the default user
    information.
    """

    voice_config: Optional[VoiceConfig]
    """Voice session configuration options."""


class Config(TypedDict, total=False):
    """
    Override the agent config for this voice session.
    Only prompt overrides are supported — voice sessions always use Gemini Live,
    so LLM model, agent model, planning prompt, and tool settings are not applicable.
    """

    custom_prompt: Optional[str]
    """Custom instructions for the AI Agent during the voice session."""

    system_prompt: Optional[str]
    """Directs your AI Agent's operational behavior during the voice session."""


class User(TypedDict, total=False):
    """User information override for converse calls.

    All fields are optional - only provided fields will override the default user information.
    """

    email: Optional[str]
    """Override the user's email for this converse call."""

    first_name: Optional[str]
    """Override the user's first name for this converse call."""

    last_name: Optional[str]
    """Override the user's last name for this converse call."""


class VoiceConfig(TypedDict, total=False):
    """Voice session configuration options."""

    input_transcription: Optional[bool]
    """Enable transcription of user input audio. Default: true."""

    output_transcription: Optional[bool]
    """Enable transcription of agent output audio. Default: true."""

    segment_max_duration_ms: Optional[float]
    """Maximum duration in milliseconds of a buffered segment before force-commit.

    Default: 180000 (3 minutes).
    """

    silence_commit_ms: Optional[float]
    """
    Duration of silence (no agent audio) in milliseconds before auto-committing a
    segment. Default: 30000 (30 seconds).
    """

    silence_discard_ratio: Optional[float]
    """Discard a segment if this fraction (0-1) of its audio is silence.

    Default: 0.9 (90% silence threshold).
    """

    voice_preset: Optional[str]
    """Voice preset to use (e.g., 'sage', 'nova', 'spark').

    If not provided, uses the agent's configured voice preset or the default.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Table"]


class Table(BaseModel):
    """The `table` object represents a table within a knowledge."""

    id: str
    """The table identifier, which can be referenced in the API endpoints."""

    created_at: datetime
    """The ISO string for when the table was created."""

    knowledge_id: str
    """The id of the knowledge this table belongs to."""

    name: str
    """The name of the table."""

    object: Literal["table"]
    """The object type, which is always `table`."""

    updated_at: datetime
    """The ISO string for when the table was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["UserUpdateParams"]


class UserUpdateParams(TypedDict, total=False):
    role: Required[Literal["admin", "member"]]
    """The role to assign to the user in the organization.

    Available roles: admin, member
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["RewriteRewriteTextParams"]


class RewriteRewriteTextParams(TypedDict, total=False):
    full_text: Required[str]
    """The full text of the document for context."""

    prompt: Required[str]
    """The prompt with instructions for rewriting the text."""

    text_to_rewrite: Required[str]
    """The text to be rewritten."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from datagrid_ai import Datagrid, AsyncDatagrid
from tests.utils import assert_matches_type
from datagrid_ai.types import VoiceSessionResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVoice:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_start_session(self, client: Datagrid) -> None:
        voice = client.voice.start_session()
        assert_matches_type(VoiceSessionResponse, voice, path=["response"])

    @parametrize
    def test_method_start_session_with_all_params(self, client: Datagrid) -> None:
        voice = client.voice.start_session(
            agent_id="agent_id",
            config={
                "custom_prompt": "custom_prompt",
                "system_prompt": "system_prompt",
            },
            conversation_id="conversation_id",
            ephemeral=True,
            file_ids=["string"],
            initial_context="initial_context",
            knowledge_ids=["string"],
            page_ids=["string"],
            secret_ids=["string"],
            user={
                "email": "email",
                "first_name": "first_name",
                "last_name": "last_name",
            },
            voice_config={
                "input_transcription": True,
                "output_transcription": True,
                "segment_max_duration_ms": 0,
                "silence_commit_ms": 0,
                "silence_discard_ratio": 0,
                "voice_preset": "voice_preset",
            },
        )
        assert_matches_type(VoiceSessionResponse, voice, path=["response"])

    @parametrize
    def test_raw_response_start_session(self, client: Datagrid) -> None:
        response = client.voice.with_raw_response.start_session()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = response.parse()
        assert_matches_type(VoiceSessionResponse, voice, path=["response"])

    @parametrize
    def test_streaming_response_start_session(self, client: Datagrid) -> None:
        with client.voice.with_streaming_response.start_session() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = response.parse()
            assert_matches_type(VoiceSessionResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncVoice:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_start_session(self, async_client: AsyncDatagrid) -> None:
        voice = await async_client.voice.start_session()
        assert_matches_type(VoiceSessionResponse, voice, path=["response"])

    @parametrize
    async def test_method_start_session_with_all_params(self, async_client: AsyncDatagrid) -> None:
        voice = await async_client.voice.start_session(
            agent_id="agent_id",
            config={
                "custom_prompt": "custom_prompt",
                "system_prompt": "system_prompt",
            },
            conversation_id="conversation_id",
            ephemeral=True,
            file_ids=["string"],
            initial_context="initial_context",
            knowledge_ids=["string"],
            page_ids=["string"],
            secret_ids=["string"],
            user={
                "email": "email",
                "first_name": "first_name",
                "last_name": "last_name",
            },
            voice_config={
                "input_transcription": True,
                "output_transcription": True,
                "segment_max_duration_ms": 0,
                "silence_commit_ms": 0,
                "silence_discard_ratio": 0,
                "voice_preset": "voice_preset",
            },
        )
        assert_matches_type(VoiceSessionResponse, voice, path=["response"])

    @parametrize
    async def test_raw_response_start_session(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.voice.with_raw_response.start_session()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = await response.parse()
        assert_matches_type(VoiceSessionResponse, voice, path=["response"])

    @parametrize
    async def test_streaming_response_start_session(self, async_client: AsyncDatagrid) -> None:
        async with async_client.voice.with_streaming_response.start_session() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = await response.parse()
            assert_matches_type(VoiceSessionResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Union, Mapping, Iterable, Optional, overload
from typing_extensions import Self, Literal, override

import httpx

from . import _exceptions
from ._qs import Querystring
from .lib import sse_converse
from .types import client_converse_params
from ._types import (
    Body,
    Omit,
    Query,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    SequenceNotStr,
    omit,
    not_given,
)
from ._utils import (
    is_given,
    maybe_transform,
    get_async_library,
    async_maybe_transform,
)
from ._compat import cached_property
from ._version import __version__
from ._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ._constants import DEFAULT_TIMEOUT
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import DatagridError, APIStatusError
from ._base_client import (
    DEFAULT_TIMEOUT,
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
    make_request_options,
)
from .types.converse_response import ConverseResponse

if TYPE_CHECKING:
    from .resources import (
        beta,
        files,
        pages,
        tools,
        voice,
        agents,
        memory,
        search,
        secrets,
        identity,
        knowledge,
        connectors,
        data_views,
        connections,
        organization,
        conversations,
        connection_providers,
    )
    from .resources.files import FilesResource, AsyncFilesResource
    from .resources.pages import PagesResource, AsyncPagesResource
    from .resources.tools import ToolsResource, AsyncToolsResource
    from .resources.voice import VoiceResource, AsyncVoiceResource
    from .resources.agents import AgentsResource, AsyncAgentsResource
    from .resources.search import SearchResource, AsyncSearchResource
    from .resources.secrets import SecretsResource, AsyncSecretsResource
    from .resources.identity import IdentityResource, AsyncIdentityResource
    from .resources.beta.beta import BetaResource, AsyncBetaResource
    from .resources.connectors import ConnectorsResource, AsyncConnectorsResource
    from .resources.connections import ConnectionsResource, AsyncConnectionsResource
    from .resources.memory.memory import MemoryResource, AsyncMemoryResource
    from .resources.knowledge.knowledge import KnowledgeResource, AsyncKnowledgeResource
    from .resources.connection_providers import ConnectionProvidersResource, AsyncConnectionProvidersResource
    from .resources.data_views.data_views import DataViewsResource, AsyncDataViewsResource
    from .resources.organization.organization import OrganizationResource, AsyncOrganizationResource
    from .resources.conversations.conversations import ConversationsResource, AsyncConversationsResource

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Datagrid",
    "AsyncDatagrid",
    "Client",
    "AsyncClient",
]


class Datagrid(SyncAPIClient):
    # client options
    api_key: str
    teamspace: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        teamspace: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Datagrid client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `DATAGRID_API_KEY`
        - `teamspace` from `DATAGRID_TEAMSPACE_ID`
        """
        if api_key is None:
            api_key = os.environ.get("DATAGRID_API_KEY")
        if api_key is None:
            raise DatagridError(
                "The api_key client option must be set either by passing api_key to the client or by setting the DATAGRID_API_KEY environment variable"
            )
        self.api_key = api_key

        if teamspace is None:
            teamspace = os.environ.get("DATAGRID_TEAMSPACE_ID")
        self.teamspace = teamspace

        if base_url is None:
            base_url = os.environ.get("DATAGRID_BASE_URL")
        if base_url is None:
            base_url = f"https://api.datagrid.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def knowledge(self) -> KnowledgeResource:
        from .resources.knowledge import KnowledgeResource

        return KnowledgeResource(self)

    @cached_property
    def connections(self) -> ConnectionsResource:
        from .resources.connections import ConnectionsResource

        return ConnectionsResource(self)

    @cached_property
    def connection_providers(self) -> ConnectionProvidersResource:
        from .resources.connection_providers import ConnectionProvidersResource

        return ConnectionProvidersResource(self)

    @cached_property
    def connectors(self) -> ConnectorsResource:
        from .resources.connectors import ConnectorsResource

        return ConnectorsResource(self)

    @cached_property
    def files(self) -> FilesResource:
        from .resources.files import FilesResource

        return FilesResource(self)

    @cached_property
    def secrets(self) -> SecretsResource:
        from .resources.secrets import SecretsResource

        return SecretsResource(self)

    @cached_property
    def search(self) -> SearchResource:
        from .resources.search import SearchResource

        return SearchResource(self)

    @cached_property
    def agents(self) -> AgentsResource:
        from .resources.agents import AgentsResource

        return AgentsResource(self)

    @cached_property
    def identity(self) -> IdentityResource:
        from .resources.identity import IdentityResource

        return IdentityResource(self)

    @cached_property
    def pages(self) -> PagesResource:
        from .resources.pages import PagesResource

        return PagesResource(self)

    @cached_property
    def tools(self) -> ToolsResource:
        from .resources.tools import ToolsResource

        return ToolsResource(self)

    @cached_property
    def memory(self) -> MemoryResource:
        from .resources.memory import MemoryResource

        return MemoryResource(self)

    @cached_property
    def organization(self) -> OrganizationResource:
        from .resources.organization import OrganizationResource

        return OrganizationResource(self)

    @cached_property
    def conversations(self) -> ConversationsResource:
        from .resources.conversations import ConversationsResource

        return ConversationsResource(self)

    @cached_property
    def data_views(self) -> DataViewsResource:
        from .resources.data_views import DataViewsResource

        return DataViewsResource(self)

    @cached_property
    def beta(self) -> BetaResource:
        from .resources.beta import BetaResource

        return BetaResource(self)

    @cached_property
    def voice(self) -> VoiceResource:
        from .resources.voice import VoiceResource

        return VoiceResource(self)

    @cached_property
    def with_raw_response(self) -> DatagridWithRawResponse:
        return DatagridWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DatagridWithStreamedResponse:
        return DatagridWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            "Datagrid-Teamspace": self.teamspace if self.teamspace is not None else Omit(),
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        teamspace: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            teamspace=teamspace or self.teamspace,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @overload
    def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: Optional[str] | Omit = omit,
        agent_routing: Optional[client_converse_params.AgentRouting] | Omit = omit,
        config: Optional[client_converse_params.Config] | Omit = omit,
        conversation_id: Optional[str] | Omit = omit,
        current_view_content: Optional[str] | Omit = omit,
        generate_citations: Optional[bool] | Omit = omit,
        include_steps: Optional[bool] | Omit = omit,
        secret_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        stream: Optional[bool] | Omit = omit,
        text: Optional[client_converse_params.Text] | Omit = omit,
        user: Optional[client_converse_params.User] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConverseResponse: ...

    @overload
    def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: Optional[str] | Omit = omit,
        agent_routing: Optional[client_converse_params.AgentRouting] | Omit = omit,
        config: Optional[client_converse_params.Config] | Omit = omit,
        conversation_id: Optional[str] | Omit = omit,
        current_view_content: Optional[str] | Omit = omit,
        generate_citations: Optional[bool] | Omit = omit,
        include_steps: Optional[bool] | Omit = omit,
        secret_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        stream: Literal[True],
        text: Optional[client_converse_params.Text] | Omit = omit,
        user: Optional[client_converse_params.User] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Stream[sse_converse.AgentStreamEvent]: ...

    @overload
    def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: Optional[str] | Omit = omit,
        agent_routing: Optional[client_converse_params.AgentRouting] | Omit = omit,
        config: Optional[client_converse_params.Config] | Omit = omit,
        conversation_id: Optional[str] | Omit = omit,
        current_view_content: Optional[str] | Omit = omit,
        generate_citations: Optional[bool] | Omit = omit,
        include_steps: Optional[bool] | Omit = omit,
        secret_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        stream: bool,
        text: Optional[client_converse_params.Text] | Omit = omit,
        user: Optional[client_converse_params.User] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConverseResponse | Stream[sse_converse.AgentStreamEvent]: ...

    def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: Optional[str] | Omit = omit,
        agent_routing: Optional[client_converse_params.AgentRouting] | Omit = omit,
        config: Optional[client_converse_params.Config] | Omit = omit,
        conversation_id: Optional[str] | Omit = omit,
        current_view_content: Optional[str] | Omit = omit,
        generate_citations: Optional[bool] | Omit = omit,
        include_steps: Optional[bool] | Omit = omit,
        secret_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        stream: Optional[bool] | Omit = omit,
        text: Optional[client_converse_params.Text] | Omit = omit,
        user: Optional[client_converse_params.User] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConverseResponse | Stream[sse_converse.AgentStreamEvent]:
        """
        Converse with an AI Agent

        Args:
          prompt: A text prompt to send to the agent.

          agent_id: The ID of the agent that should be used for the converse. When omitted and a
              `conversation_id` is provided, the conversation's existing agent assignments are
              preserved. When omitted without a `conversation_id`, a new conversation is
              created with the default agent.

          agent_routing: Determines how the API routes the converse request to an agent.

          config: Override the agent config for this converse call. This is applied as a partial
              override.

          conversation_id: The ID of the present conversation to use. If it's not provided - a new
              conversation will be created.

          current_view_content: A datagrid file URI pointing to content the user is currently viewing on screen
              (e.g., a web page, document, or dashboard rendered as markdown). The agent uses
              this context to resolve ambiguous queries like 'what is this about?' or 'review
              this'. The content is automatically summarized and made available to the agent.

          generate_citations: Determines whether the response should include citations. When enabled, the
              agent will generate citations for factual statements.

          include_steps: When set to false, tool call and reasoning step events are omitted from SSE
              streams. Non-streaming responses always include the tool_calls and reasoning
              fields (as null when empty).

          secret_ids: Array of secret ID's to be included in the context. The secret value will be
              appended to the prompt but not stored in conversation history.

          stream: Determines the response type of the converse. Response is the Server-Sent Events
              if stream is set to true.

          text: Contains the format property used to specify the structured output schema.
              Structured output is supported by the following agent models: `magpie-2.0`
              (default), `magpie-2.5`, and `magpie-1.1`. It is not supported by
              `magpie-1.1-flash` (Ask mode) or `llm-only` (Fastest mode).

          user: User information override for converse calls. All fields are optional - only
              provided fields will override the default user information.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not is_given(timeout) and self.timeout == DEFAULT_TIMEOUT:
            timeout = 1800
        return self.post(
            "/converse",
            body=maybe_transform(
                {
                    "prompt": prompt,
                    "agent_id": agent_id,
                    "agent_routing": agent_routing,
                    "config": config,
                    "conversation_id": conversation_id,
                    "current_view_content": current_view_content,
                    "generate_citations": generate_citations,
                    "include_steps": include_steps,
                    "secret_ids": secret_ids,
                    "stream": stream,
                    "text": text,
                    "user": user,
                },
                client_converse_params.ClientConverseParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConverseResponse,
            stream=stream or False,
            stream_cls=Stream[sse_converse.AgentStreamEvent],
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncDatagrid(AsyncAPIClient):
    # client options
    api_key: str
    teamspace: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        teamspace: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncDatagrid client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `DATAGRID_API_KEY`
        - `teamspace` from `DATAGRID_TEAMSPACE_ID`
        """
        if api_key is None:
            api_key = os.environ.get("DATAGRID_API_KEY")
        if api_key is None:
            raise DatagridError(
                "The api_key client option must be set either by passing api_key to the client or by setting the DATAGRID_API_KEY environment variable"
            )
        self.api_key = api_key

        if teamspace is None:
            teamspace = os.environ.get("DATAGRID_TEAMSPACE_ID")
        self.teamspace = teamspace

        if base_url is None:
            base_url = os.environ.get("DATAGRID_BASE_URL")
        if base_url is None:
            base_url = f"https://api.datagrid.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def knowledge(self) -> AsyncKnowledgeResource:
        from .resources.knowledge import AsyncKnowledgeResource

        return AsyncKnowledgeResource(self)

    @cached_property
    def connections(self) -> AsyncConnectionsResource:
        from .resources.connections import AsyncConnectionsResource

        return AsyncConnectionsResource(self)

    @cached_property
    def connection_providers(self) -> AsyncConnectionProvidersResource:
        from .resources.connection_providers import AsyncConnectionProvidersResource

        return AsyncConnectionProvidersResource(self)

    @cached_property
    def connectors(self) -> AsyncConnectorsResource:
        from .resources.connectors import AsyncConnectorsResource

        return AsyncConnectorsResource(self)

    @cached_property
    def files(self) -> AsyncFilesResource:
        from .resources.files import AsyncFilesResource

        return AsyncFilesResource(self)

    @cached_property
    def secrets(self) -> AsyncSecretsResource:
        from .resources.secrets import AsyncSecretsResource

        return AsyncSecretsResource(self)

    @cached_property
    def search(self) -> AsyncSearchResource:
        from .resources.search import AsyncSearchResource

        return AsyncSearchResource(self)

    @cached_property
    def agents(self) -> AsyncAgentsResource:
        from .resources.agents import AsyncAgentsResource

        return AsyncAgentsResource(self)

    @cached_property
    def identity(self) -> AsyncIdentityResource:
        from .resources.identity import AsyncIdentityResource

        return AsyncIdentityResource(self)

    @cached_property
    def pages(self) -> AsyncPagesResource:
        from .resources.pages import AsyncPagesResource

        return AsyncPagesResource(self)

    @cached_property
    def tools(self) -> AsyncToolsResource:
        from .resources.tools import AsyncToolsResource

        return AsyncToolsResource(self)

    @cached_property
    def memory(self) -> AsyncMemoryResource:
        from .resources.memory import AsyncMemoryResource

        return AsyncMemoryResource(self)

    @cached_property
    def organization(self) -> AsyncOrganizationResource:
        from .resources.organization import AsyncOrganizationResource

        return AsyncOrganizationResource(self)

    @cached_property
    def conversations(self) -> AsyncConversationsResource:
        from .resources.conversations import AsyncConversationsResource

        return AsyncConversationsResource(self)

    @cached_property
    def data_views(self) -> AsyncDataViewsResource:
        from .resources.data_views import AsyncDataViewsResource

        return AsyncDataViewsResource(self)

    @cached_property
    def beta(self) -> AsyncBetaResource:
        from .resources.beta import AsyncBetaResource

        return AsyncBetaResource(self)

    @cached_property
    def voice(self) -> AsyncVoiceResource:
        from .resources.voice import AsyncVoiceResource

        return AsyncVoiceResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncDatagridWithRawResponse:
        return AsyncDatagridWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDatagridWithStreamedResponse:
        return AsyncDatagridWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            "Datagrid-Teamspace": self.teamspace if self.teamspace is not None else Omit(),
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        teamspace: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            teamspace=teamspace or self.teamspace,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @overload
    async def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: Optional[str] | Omit = omit,
        agent_routing: Optional[client_converse_params.AgentRouting] | Omit = omit,
        config: Optional[client_converse_params.Config] | Omit = omit,
        conversation_id: Optional[str] | Omit = omit,
        current_view_content: Optional[str] | Omit = omit,
        generate_citations: Optional[bool] | Omit = omit,
        include_steps: Optional[bool] | Omit = omit,
        secret_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        stream: Literal[False] | Omit = omit,
        text: Optional[client_converse_params.Text] | Omit = omit,
        user: Optional[client_converse_params.User] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConverseResponse: ...

    @overload
    async def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: Optional[str] | Omit = omit,
        agent_routing: Optional[client_converse_params.AgentRouting] | Omit = omit,
        config: Optional[client_converse_params.Config] | Omit = omit,
        conversation_id: Optional[str] | Omit = omit,
        current_view_content: Optional[str] | Omit = omit,
        generate_citations: Optional[bool] | Omit = omit,
        include_steps: Optional[bool] | Omit = omit,
        secret_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        stream: Literal[True],
        text: Optional[client_converse_params.Text] | Omit = omit,
        user: Optional[client_converse_params.User] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncStream[sse_converse.AgentStreamEvent]: ...

    @overload
    async def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: Optional[str] | Omit = omit,
        agent_routing: Optional[client_converse_params.AgentRouting] | Omit = omit,
        config: Optional[client_converse_params.Config] | Omit = omit,
        conversation_id: Optional[str] | Omit = omit,
        current_view_content: Optional[str] | Omit = omit,
        generate_citations: Optional[bool] | Omit = omit,
        include_steps: Optional[bool] | Omit = omit,
        secret_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        stream: bool,
        text: Optional[client_converse_params.Text] | Omit = omit,
        user: Optional[client_converse_params.User] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConverseResponse | AsyncStream[sse_converse.AgentStreamEvent]: ...

    async def converse(
        self,
        *,
        prompt: Union[str, Iterable[client_converse_params.PromptInputItemList]],
        agent_id: Optional[str] | Omit = omit,
        agent_routing: Optional[client_converse_params.AgentRouting] | Omit = omit,
        config: Optional[client_converse_params.Config] | Omit = omit,
        conversation_id: Optional[str] | Omit = omit,
        current_view_content: Optional[str] | Omit = omit,
        generate_citations: Optional[bool] | Omit = omit,
        include_steps: Optional[bool] | Omit = omit,
        secret_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        stream: Optional[bool] | Omit = omit,
        text: Optional[client_converse_params.Text] | Omit = omit,
        user: Optional[client_converse_params.User] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConverseResponse | AsyncStream[sse_converse.AgentStreamEvent]:
        """
        Converse with an AI Agent

        Args:
          prompt: A text prompt to send to the agent.

          agent_id: The ID of the agent that should be used for the converse. When omitted and a
              `conversation_id` is provided, the conversation's existing agent assignments are
              preserved. When omitted without a `conversation_id`, a new conversation is
              created with the default agent.

          agent_routing: Determines how the API routes the converse request to an agent.

          config: Override the agent config for this converse call. This is applied as a partial
              override.

          conversation_id: The ID of the present conversation to use. If it's not provided - a new
              conversation will be created.

          current_view_content: A datagrid file URI pointing to content the user is currently viewing on screen
              (e.g., a web page, document, or dashboard rendered as markdown). The agent uses
              this context to resolve ambiguous queries like 'what is this about?' or 'review
              this'. The content is automatically summarized and made available to the agent.

          generate_citations: Determines whether the response should include citations. When enabled, the
              agent will generate citations for factual statements.

          include_steps: When set to false, tool call and reasoning step events are omitted from SSE
              streams. Non-streaming responses always include the tool_calls and reasoning
              fields (as null when empty).

          secret_ids: Array of secret ID's to be included in the context. The secret value will be
              appended to the prompt but not stored in conversation history.

          stream: Determines the response type of the converse. Response is the Server-Sent Events
              if stream is set to true.

          text: Contains the format property used to specify the structured output schema.
              Structured output is supported by the following agent models: `magpie-2.0`
              (default), `magpie-2.5`, and `magpie-1.1`. It is not supported by
              `magpie-1.1-flash` (Ask mode) or `llm-only` (Fastest mode).

          user: User information override for converse calls. All fields are optional - only
              provided fields will override the default user information.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not is_given(timeout) and self.timeout == DEFAULT_TIMEOUT:
            timeout = 1800
        return await self.post(
            "/converse",
            body=await async_maybe_transform(
                {
                    "prompt": prompt,
                    "agent_id": agent_id,
                    "agent_routing": agent_routing,
                    "config": config,
                    "conversation_id": conversation_id,
                    "current_view_content": current_view_content,
                    "generate_citations": generate_citations,
                    "include_steps": include_steps,
                    "secret_ids": secret_ids,
                    "stream": stream,
                    "text": text,
                    "user": user,
                },
                client_converse_params.ClientConverseParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConverseResponse,
            stream=stream or False,
            stream_cls=AsyncStream[sse_converse.AgentStreamEvent],
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class DatagridWithRawResponse:
    _client: Datagrid

    def __init__(self, client: Datagrid) -> None:
        self._client = client

        self.converse = to_raw_response_wrapper(
            client.converse,
        )

    @cached_property
    def knowledge(self) -> knowledge.KnowledgeResourceWithRawResponse:
        from .resources.knowledge import KnowledgeResourceWithRawResponse

        return KnowledgeResourceWithRawResponse(self._client.knowledge)

    @cached_property
    def connections(self) -> connections.ConnectionsResourceWithRawResponse:
        from .resources.connections import ConnectionsResourceWithRawResponse

        return ConnectionsResourceWithRawResponse(self._client.connections)

    @cached_property
    def connection_providers(self) -> connection_providers.ConnectionProvidersResourceWithRawResponse:
        from .resources.connection_providers import ConnectionProvidersResourceWithRawResponse

        return ConnectionProvidersResourceWithRawResponse(self._client.connection_providers)

    @cached_property
    def connectors(self) -> connectors.ConnectorsResourceWithRawResponse:
        from .resources.connectors import ConnectorsResourceWithRawResponse

        return ConnectorsResourceWithRawResponse(self._client.connectors)

    @cached_property
    def files(self) -> files.FilesResourceWithRawResponse:
        from .resources.files import FilesResourceWithRawResponse

        return FilesResourceWithRawResponse(self._client.files)

    @cached_property
    def secrets(self) -> secrets.SecretsResourceWithRawResponse:
        from .resources.secrets import SecretsResourceWithRawResponse

        return SecretsResourceWithRawResponse(self._client.secrets)

    @cached_property
    def search(self) -> search.SearchResourceWithRawResponse:
        from .resources.search import SearchResourceWithRawResponse

        return SearchResourceWithRawResponse(self._client.search)

    @cached_property
    def agents(self) -> agents.AgentsResourceWithRawResponse:
        from .resources.agents import AgentsResourceWithRawResponse

        return AgentsResourceWithRawResponse(self._client.agents)

    @cached_property
    def identity(self) -> identity.IdentityResourceWithRawResponse:
        from .resources.identity import IdentityResourceWithRawResponse

        return IdentityResourceWithRawResponse(self._client.identity)

    @cached_property
    def pages(self) -> pages.PagesResourceWithRawResponse:
        from .resources.pages import PagesResourceWithRawResponse

        return PagesResourceWithRawResponse(self._client.pages)

    @cached_property
    def tools(self) -> tools.ToolsResourceWithRawResponse:
        from .resources.tools import ToolsResourceWithRawResponse

        return ToolsResourceWithRawResponse(self._client.tools)

    @cached_property
    def memory(self) -> memory.MemoryResourceWithRawResponse:
        from .resources.memory import MemoryResourceWithRawResponse

        return MemoryResourceWithRawResponse(self._client.memory)

    @cached_property
    def organization(self) -> organization.OrganizationResourceWithRawResponse:
        from .resources.organization import OrganizationResourceWithRawResponse

        return OrganizationResourceWithRawResponse(self._client.organization)

    @cached_property
    def conversations(self) -> conversations.ConversationsResourceWithRawResponse:
        from .resources.conversations import ConversationsResourceWithRawResponse

        return ConversationsResourceWithRawResponse(self._client.conversations)

    @cached_property
    def data_views(self) -> data_views.DataViewsResourceWithRawResponse:
        from .resources.data_views import DataViewsResourceWithRawResponse

        return DataViewsResourceWithRawResponse(self._client.data_views)

    @cached_property
    def beta(self) -> beta.BetaResourceWithRawResponse:
        from .resources.beta import BetaResourceWithRawResponse

        return BetaResourceWithRawResponse(self._client.beta)

    @cached_property
    def voice(self) -> voice.VoiceResourceWithRawResponse:
        from .resources.voice import VoiceResourceWithRawResponse

        return VoiceResourceWithRawResponse(self._client.voice)


class AsyncDatagridWithRawResponse:
    _client: AsyncDatagrid

    def __init__(self, client: AsyncDatagrid) -> None:
        self._client = client

        self.converse = async_to_raw_response_wrapper(
            client.converse,
        )

    @cached_property
    def knowledge(self) -> knowledge.AsyncKnowledgeResourceWithRawResponse:
        from .resources.knowledge import AsyncKnowledgeResourceWithRawResponse

        return AsyncKnowledgeResourceWithRawResponse(self._client.knowledge)

    @cached_property
    def connections(self) -> connections.AsyncConnectionsResourceWithRawResponse:
        from .resources.connections import AsyncConnectionsResourceWithRawResponse

        return AsyncConnectionsResourceWithRawResponse(self._client.connections)

    @cached_property
    def connection_providers(self) -> connection_providers.AsyncConnectionProvidersResourceWithRawResponse:
        from .resources.connection_providers import AsyncConnectionProvidersResourceWithRawResponse

        return AsyncConnectionProvidersResourceWithRawResponse(self._client.connection_providers)

    @cached_property
    def connectors(self) -> connectors.AsyncConnectorsResourceWithRawResponse:
        from .resources.connectors import AsyncConnectorsResourceWithRawResponse

        return AsyncConnectorsResourceWithRawResponse(self._client.connectors)

    @cached_property
    def files(self) -> files.AsyncFilesResourceWithRawResponse:
        from .resources.files import AsyncFilesResourceWithRawResponse

        return AsyncFilesResourceWithRawResponse(self._client.files)

    @cached_property
    def secrets(self) -> secrets.AsyncSecretsResourceWithRawResponse:
        from .resources.secrets import AsyncSecretsResourceWithRawResponse

        return AsyncSecretsResourceWithRawResponse(self._client.secrets)

    @cached_property
    def search(self) -> search.AsyncSearchResourceWithRawResponse:
        from .resources.search import AsyncSearchResourceWithRawResponse

        return AsyncSearchResourceWithRawResponse(self._client.search)

    @cached_property
    def agents(self) -> agents.AsyncAgentsResourceWithRawResponse:
        from .resources.agents import AsyncAgentsResourceWithRawResponse

        return AsyncAgentsResourceWithRawResponse(self._client.agents)

    @cached_property
    def identity(self) -> identity.AsyncIdentityResourceWithRawResponse:
        from .resources.identity import AsyncIdentityResourceWithRawResponse

        return AsyncIdentityResourceWithRawResponse(self._client.identity)

    @cached_property
    def pages(self) -> pages.AsyncPagesResourceWithRawResponse:
        from .resources.pages import AsyncPagesResourceWithRawResponse

        return AsyncPagesResourceWithRawResponse(self._client.pages)

    @cached_property
    def tools(self) -> tools.AsyncToolsResourceWithRawResponse:
        from .resources.tools import AsyncToolsResourceWithRawResponse

        return AsyncToolsResourceWithRawResponse(self._client.tools)

    @cached_property
    def memory(self) -> memory.AsyncMemoryResourceWithRawResponse:
        from .resources.memory import AsyncMemoryResourceWithRawResponse

        return AsyncMemoryResourceWithRawResponse(self._client.memory)

    @cached_property
    def organization(self) -> organization.AsyncOrganizationResourceWithRawResponse:
        from .resources.organization import AsyncOrganizationResourceWithRawResponse

        return AsyncOrganizationResourceWithRawResponse(self._client.organization)

    @cached_property
    def conversations(self) -> conversations.AsyncConversationsResourceWithRawResponse:
        from .resources.conversations import AsyncConversationsResourceWithRawResponse

        return AsyncConversationsResourceWithRawResponse(self._client.conversations)

    @cached_property
    def data_views(self) -> data_views.AsyncDataViewsResourceWithRawResponse:
        from .resources.data_views import AsyncDataViewsResourceWithRawResponse

        return AsyncDataViewsResourceWithRawResponse(self._client.data_views)

    @cached_property
    def beta(self) -> beta.AsyncBetaResourceWithRawResponse:
        from .resources.beta import AsyncBetaResourceWithRawResponse

        return AsyncBetaResourceWithRawResponse(self._client.beta)

    @cached_property
    def voice(self) -> voice.AsyncVoiceResourceWithRawResponse:
        from .resources.voice import AsyncVoiceResourceWithRawResponse

        return AsyncVoiceResourceWithRawResponse(self._client.voice)


class DatagridWithStreamedResponse:
    _client: Datagrid

    def __init__(self, client: Datagrid) -> None:
        self._client = client

        self.converse = to_streamed_response_wrapper(
            client.converse,
        )

    @cached_property
    def knowledge(self) -> knowledge.KnowledgeResourceWithStreamingResponse:
        from .resources.knowledge import KnowledgeResourceWithStreamingResponse

        return KnowledgeResourceWithStreamingResponse(self._client.knowledge)

    @cached_property
    def connections(self) -> connections.ConnectionsResourceWithStreamingResponse:
        from .resources.connections import ConnectionsResourceWithStreamingResponse

        return ConnectionsResourceWithStreamingResponse(self._client.connections)

    @cached_property
    def connection_providers(self) -> connection_providers.ConnectionProvidersResourceWithStreamingResponse:
        from .resources.connection_providers import ConnectionProvidersResourceWithStreamingResponse

        return ConnectionProvidersResourceWithStreamingResponse(self._client.connection_providers)

    @cached_property
    def connectors(self) -> connectors.ConnectorsResourceWithStreamingResponse:
        from .resources.connectors import ConnectorsResourceWithStreamingResponse

        return ConnectorsResourceWithStreamingResponse(self._client.connectors)

    @cached_property
    def files(self) -> files.FilesResourceWithStreamingResponse:
        from .resources.files import FilesResourceWithStreamingResponse

        return FilesResourceWithStreamingResponse(self._client.files)

    @cached_property
    def secrets(self) -> secrets.SecretsResourceWithStreamingResponse:
        from .resources.secrets import SecretsResourceWithStreamingResponse

        return SecretsResourceWithStreamingResponse(self._client.secrets)

    @cached_property
    def search(self) -> search.SearchResourceWithStreamingResponse:
        from .resources.search import SearchResourceWithStreamingResponse

        return SearchResourceWithStreamingResponse(self._client.search)

    @cached_property
    def agents(self) -> agents.AgentsResourceWithStreamingResponse:
        from .resources.agents import AgentsResourceWithStreamingResponse

        return AgentsResourceWithStreamingResponse(self._client.agents)

    @cached_property
    def identity(self) -> identity.IdentityResourceWithStreamingResponse:
        from .resources.identity import IdentityResourceWithStreamingResponse

        return IdentityResourceWithStreamingResponse(self._client.identity)

    @cached_property
    def pages(self) -> pages.PagesResourceWithStreamingResponse:
        from .resources.pages import PagesResourceWithStreamingResponse

        return PagesResourceWithStreamingResponse(self._client.pages)

    @cached_property
    def tools(self) -> tools.ToolsResourceWithStreamingResponse:
        from .resources.tools import ToolsResourceWithStreamingResponse

        return ToolsResourceWithStreamingResponse(self._client.tools)

    @cached_property
    def memory(self) -> memory.MemoryResourceWithStreamingResponse:
        from .resources.memory import MemoryResourceWithStreamingResponse

        return MemoryResourceWithStreamingResponse(self._client.memory)

    @cached_property
    def organization(self) -> organization.OrganizationResourceWithStreamingResponse:
        from .resources.organization import OrganizationResourceWithStreamingResponse

        return OrganizationResourceWithStreamingResponse(self._client.organization)

    @cached_property
    def conversations(self) -> conversations.ConversationsResourceWithStreamingResponse:
        from .resources.conversations import ConversationsResourceWithStreamingResponse

        return ConversationsResourceWithStreamingResponse(self._client.conversations)

    @cached_property
    def data_views(self) -> data_views.DataViewsResourceWithStreamingResponse:
        from .resources.data_views import DataViewsResourceWithStreamingResponse

        return DataViewsResourceWithStreamingResponse(self._client.data_views)

    @cached_property
    def beta(self) -> beta.BetaResourceWithStreamingResponse:
        from .resources.beta import BetaResourceWithStreamingResponse

        return BetaResourceWithStreamingResponse(self._client.beta)

    @cached_property
    def voice(self) -> voice.VoiceResourceWithStreamingResponse:
        from .resources.voice import VoiceResourceWithStreamingResponse

        return VoiceResourceWithStreamingResponse(self._client.voice)


class AsyncDatagridWithStreamedResponse:
    _client: AsyncDatagrid

    def __init__(self, client: AsyncDatagrid) -> None:
        self._client = client

        self.converse = async_to_streamed_response_wrapper(
            client.converse,
        )

    @cached_property
    def knowledge(self) -> knowledge.AsyncKnowledgeResourceWithStreamingResponse:
        from .resources.knowledge import AsyncKnowledgeResourceWithStreamingResponse

        return AsyncKnowledgeResourceWithStreamingResponse(self._client.knowledge)

    @cached_property
    def connections(self) -> connections.AsyncConnectionsResourceWithStreamingResponse:
        from .resources.connections import AsyncConnectionsResourceWithStreamingResponse

        return AsyncConnectionsResourceWithStreamingResponse(self._client.connections)

    @cached_property
    def connection_providers(self) -> connection_providers.AsyncConnectionProvidersResourceWithStreamingResponse:
        from .resources.connection_providers import AsyncConnectionProvidersResourceWithStreamingResponse

        return AsyncConnectionProvidersResourceWithStreamingResponse(self._client.connection_providers)

    @cached_property
    def connectors(self) -> connectors.AsyncConnectorsResourceWithStreamingResponse:
        from .resources.connectors import AsyncConnectorsResourceWithStreamingResponse

        return AsyncConnectorsResourceWithStreamingResponse(self._client.connectors)

    @cached_property
    def files(self) -> files.AsyncFilesResourceWithStreamingResponse:
        from .resources.files import AsyncFilesResourceWithStreamingResponse

        return AsyncFilesResourceWithStreamingResponse(self._client.files)

    @cached_property
    def secrets(self) -> secrets.AsyncSecretsResourceWithStreamingResponse:
        from .resources.secrets import AsyncSecretsResourceWithStreamingResponse

        return AsyncSecretsResourceWithStreamingResponse(self._client.secrets)

    @cached_property
    def search(self) -> search.AsyncSearchResourceWithStreamingResponse:
        from .resources.search import AsyncSearchResourceWithStreamingResponse

        return AsyncSearchResourceWithStreamingResponse(self._client.search)

    @cached_property
    def agents(self) -> agents.AsyncAgentsResourceWithStreamingResponse:
        from .resources.agents import AsyncAgentsResourceWithStreamingResponse

        return AsyncAgentsResourceWithStreamingResponse(self._client.agents)

    @cached_property
    def identity(self) -> identity.AsyncIdentityResourceWithStreamingResponse:
        from .resources.identity import AsyncIdentityResourceWithStreamingResponse

        return AsyncIdentityResourceWithStreamingResponse(self._client.identity)

    @cached_property
    def pages(self) -> pages.AsyncPagesResourceWithStreamingResponse:
        from .resources.pages import AsyncPagesResourceWithStreamingResponse

        return AsyncPagesResourceWithStreamingResponse(self._client.pages)

    @cached_property
    def tools(self) -> tools.AsyncToolsResourceWithStreamingResponse:
        from .resources.tools import AsyncToolsResourceWithStreamingResponse

        return AsyncToolsResourceWithStreamingResponse(self._client.tools)

    @cached_property
    def memory(self) -> memory.AsyncMemoryResourceWithStreamingResponse:
        from .resources.memory import AsyncMemoryResourceWithStreamingResponse

        return AsyncMemoryResourceWithStreamingResponse(self._client.memory)

    @cached_property
    def organization(self) -> organization.AsyncOrganizationResourceWithStreamingResponse:
        from .resources.organization import AsyncOrganizationResourceWithStreamingResponse

        return AsyncOrganizationResourceWithStreamingResponse(self._client.organization)

    @cached_property
    def conversations(self) -> conversations.AsyncConversationsResourceWithStreamingResponse:
        from .resources.conversations import AsyncConversationsResourceWithStreamingResponse

        return AsyncConversationsResourceWithStreamingResponse(self._client.conversations)

    @cached_property
    def data_views(self) -> data_views.AsyncDataViewsResourceWithStreamingResponse:
        from .resources.data_views import AsyncDataViewsResourceWithStreamingResponse

        return AsyncDataViewsResourceWithStreamingResponse(self._client.data_views)

    @cached_property
    def beta(self) -> beta.AsyncBetaResourceWithStreamingResponse:
        from .resources.beta import AsyncBetaResourceWithStreamingResponse

        return AsyncBetaResourceWithStreamingResponse(self._client.beta)

    @cached_property
    def voice(self) -> voice.AsyncVoiceResourceWithStreamingResponse:
        from .resources.voice import AsyncVoiceResourceWithStreamingResponse

        return AsyncVoiceResourceWithStreamingResponse(self._client.voice)


Client = Datagrid

AsyncClient = AsyncDatagrid



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from datagrid_ai import Datagrid, AsyncDatagrid
from tests.utils import assert_matches_type
from datagrid_ai.types import (
    SearchAIResult,
    SearchTreeResult,
    SearchSearchResponse,
)

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSearch:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_search(self, client: Datagrid) -> None:
        with pytest.warns(DeprecationWarning):
            search = client.search.search(
                query="x",
            )

        assert_matches_type(SearchSearchResponse, search, path=["response"])

    @parametrize
    def test_method_search_with_all_params(self, client: Datagrid) -> None:
        with pytest.warns(DeprecationWarning):
            search = client.search.search(
                query="x",
                knowledge_ids=["string"],
                limit=1,
                next="next",
                record_types=["rows"],
            )

        assert_matches_type(SearchSearchResponse, search, path=["response"])

    @parametrize
    def test_raw_response_search(self, client: Datagrid) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.search.with_raw_response.search(
                query="x",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = response.parse()
        assert_matches_type(SearchSearchResponse, search, path=["response"])

    @parametrize
    def test_streaming_response_search(self, client: Datagrid) -> None:
        with pytest.warns(DeprecationWarning):
            with client.search.with_streaming_response.search(
                query="x",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                search = response.parse()
                assert_matches_type(SearchSearchResponse, search, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_search_ai(self, client: Datagrid) -> None:
        search = client.search.search_ai(
            query="query",
        )
        assert_matches_type(SearchAIResult, search, path=["response"])

    @parametrize
    def test_method_search_ai_with_all_params(self, client: Datagrid) -> None:
        search = client.search.search_ai(
            query="query",
            limit=0,
            record_types=["rows"],
        )
        assert_matches_type(SearchAIResult, search, path=["response"])

    @parametrize
    def test_raw_response_search_ai(self, client: Datagrid) -> None:
        response = client.search.with_raw_response.search_ai(
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = response.parse()
        assert_matches_type(SearchAIResult, search, path=["response"])

    @parametrize
    def test_streaming_response_search_ai(self, client: Datagrid) -> None:
        with client.search.with_streaming_response.search_ai(
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = response.parse()
            assert_matches_type(SearchAIResult, search, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_search_tree(self, client: Datagrid) -> None:
        search = client.search.search_tree(
            query="query",
        )
        assert_matches_type(SearchTreeResult, search, path=["response"])

    @parametrize
    def test_method_search_tree_with_all_params(self, client: Datagrid) -> None:
        search = client.search.search_tree(
            query="query",
            limit=1,
            next="next",
            record_types=["rows"],
        )
        assert_matches_type(SearchTreeResult, search, path=["response"])

    @parametrize
    def test_raw_response_search_tree(self, client: Datagrid) -> None:
        response = client.search.with_raw_response.search_tree(
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = response.parse()
        assert_matches_type(SearchTreeResult, search, path=["response"])

    @parametrize
    def test_streaming_response_search_tree(self, client: Datagrid) -> None:
        with client.search.with_streaming_response.search_tree(
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = response.parse()
            assert_matches_type(SearchTreeResult, search, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSearch:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_search(self, async_client: AsyncDatagrid) -> None:
        with pytest.warns(DeprecationWarning):
            search = await async_client.search.search(
                query="x",
            )

        assert_matches_type(SearchSearchResponse, search, path=["response"])

    @parametrize
    async def test_method_search_with_all_params(self, async_client: AsyncDatagrid) -> None:
        with pytest.warns(DeprecationWarning):
            search = await async_client.search.search(
                query="x",
                knowledge_ids=["string"],
                limit=1,
                next="next",
                record_types=["rows"],
            )

        assert_matches_type(SearchSearchResponse, search, path=["response"])

    @parametrize
    async def test_raw_response_search(self, async_client: AsyncDatagrid) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.search.with_raw_response.search(
                query="x",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = await response.parse()
        assert_matches_type(SearchSearchResponse, search, path=["response"])

    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncDatagrid) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.search.with_streaming_response.search(
                query="x",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                search = await response.parse()
                assert_matches_type(SearchSearchResponse, search, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_search_ai(self, async_client: AsyncDatagrid) -> None:
        search = await async_client.search.search_ai(
            query="query",
        )
        assert_matches_type(SearchAIResult, search, path=["response"])

    @parametrize
    async def test_method_search_ai_with_all_params(self, async_client: AsyncDatagrid) -> None:
        search = await async_client.search.search_ai(
            query="query",
            limit=0,
            record_types=["rows"],
        )
        assert_matches_type(SearchAIResult, search, path=["response"])

    @parametrize
    async def test_raw_response_search_ai(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.search.with_raw_response.search_ai(
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = await response.parse()
        assert_matches_type(SearchAIResult, search, path=["response"])

    @parametrize
    async def test_streaming_response_search_ai(self, async_client: AsyncDatagrid) -> None:
        async with async_client.search.with_streaming_response.search_ai(
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = await response.parse()
            assert_matches_type(SearchAIResult, search, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_search_tree(self, async_client: AsyncDatagrid) -> None:
        search = await async_client.search.search_tree(
            query="query",
        )
        assert_matches_type(SearchTreeResult, search, path=["response"])

    @parametrize
    async def test_method_search_tree_with_all_params(self, async_client: AsyncDatagrid) -> None:
        search = await async_client.search.search_tree(
            query="query",
            limit=1,
            next="next",
            record_types=["rows"],
        )
        assert_matches_type(SearchTreeResult, search, path=["response"])

    @parametrize
    async def test_raw_response_search_tree(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.search.with_raw_response.search_tree(
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = await response.parse()
        assert_matches_type(SearchTreeResult, search, path=["response"])

    @parametrize
    async def test_streaming_response_search_tree(self, async_client: AsyncDatagrid) -> None:
        async with async_client.search.with_streaming_response.search_tree(
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = await response.parse()
            assert_matches_type(SearchTreeResult, search, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ConnectionProvider"]


class ConnectionProvider(BaseModel):
    """
    The `ConnectionProvider` object represents custom OAuth credentials for a connector. When creating a connection with a connection provider, the specified client ID and secret will be used instead of the default credentials.
    """

    id: str
    """The connection provider identifier."""

    client_id: str
    """The OAuth client ID to use for this connector."""

    client_secret_id: str
    """
    The ID of the secret containing the OAuth client secret to use for this
    connector.
    """

    connector_id: str
    """The connector ID this provider is configured for."""

    created_at: datetime
    """The date and time the connection provider was created."""

    name: str
    """The name of the connection provider."""

    object: Literal["connection_provider"]
    """The object type, which is always `connection_provider`."""

    updated_at: datetime
    """The date and time the connection provider was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.organization import mcp_server_create_params, mcp_server_update_params
from ...types.organization.mcp_server import McpServer
from ...types.organization.list_mcp_servers_response import ListMcpServersResponse

__all__ = ["McpServersResource", "AsyncMcpServersResource"]


class McpServersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> McpServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return McpServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> McpServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return McpServersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        base_url: str,
        name: str,
        authorization: Optional[str] | Omit = omit,
        authorization_secret_id: Optional[str] | Omit = omit,
        icon_url: Optional[str] | Omit = omit,
        protocol_version: Optional[str] | Omit = omit,
        transport: Optional[Literal["http"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServer:
        """
        Register a new MCP server in the current teamspace.

        Args:
          base_url: The HTTPS URL of the MCP server.

          authorization: Raw Authorization header value (for example, 'Bearer <token>'). Datagrid stores
              it as a secret and links it to this server. If both authorization and
              authorization_secret_id are provided, authorization takes precedence.

          authorization_secret_id: Secret ID containing the full Authorization header value to use when calling
              this MCP server.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/organization/mcp-servers",
            body=maybe_transform(
                {
                    "base_url": base_url,
                    "name": name,
                    "authorization": authorization,
                    "authorization_secret_id": authorization_secret_id,
                    "icon_url": icon_url,
                    "protocol_version": protocol_version,
                    "transport": transport,
                },
                mcp_server_create_params.McpServerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServer,
        )

    def retrieve(
        self,
        server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServer:
        """
        Retrieve a registered MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not server_id:
            raise ValueError(f"Expected a non-empty value for `server_id` but received {server_id!r}")
        return self._get(
            path_template("/organization/mcp-servers/{server_id}", server_id=server_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServer,
        )

    def update(
        self,
        server_id: str,
        *,
        authorization: Optional[str] | Omit = omit,
        authorization_secret_id: Optional[str] | Omit = omit,
        base_url: str | Omit = omit,
        icon_url: Optional[str] | Omit = omit,
        name: str | Omit = omit,
        protocol_version: Optional[str] | Omit = omit,
        transport: Optional[Literal["http"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServer:
        """
        Update a registered MCP server.

        Args:
          authorization: Raw Authorization header value (for example, 'Bearer <token>'). Datagrid stores
              it as a secret and links it to this server. Set to null to clear. If both
              authorization and authorization_secret_id are provided, authorization takes
              precedence.

          authorization_secret_id: Secret ID containing the full Authorization header value to use when calling
              this MCP server. Set to null to clear.

          base_url: The HTTPS URL of the MCP server.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not server_id:
            raise ValueError(f"Expected a non-empty value for `server_id` but received {server_id!r}")
        return self._patch(
            path_template("/organization/mcp-servers/{server_id}", server_id=server_id),
            body=maybe_transform(
                {
                    "authorization": authorization,
                    "authorization_secret_id": authorization_secret_id,
                    "base_url": base_url,
                    "icon_url": icon_url,
                    "name": name,
                    "protocol_version": protocol_version,
                    "transport": transport,
                },
                mcp_server_update_params.McpServerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServer,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ListMcpServersResponse:
        """List registered MCP servers for the current teamspace."""
        return self._get(
            "/organization/mcp-servers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ListMcpServersResponse,
        )

    def delete(
        self,
        server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a registered MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not server_id:
            raise ValueError(f"Expected a non-empty value for `server_id` but received {server_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template("/organization/mcp-servers/{server_id}", server_id=server_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncMcpServersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMcpServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMcpServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMcpServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return AsyncMcpServersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        base_url: str,
        name: str,
        authorization: Optional[str] | Omit = omit,
        authorization_secret_id: Optional[str] | Omit = omit,
        icon_url: Optional[str] | Omit = omit,
        protocol_version: Optional[str] | Omit = omit,
        transport: Optional[Literal["http"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServer:
        """
        Register a new MCP server in the current teamspace.

        Args:
          base_url: The HTTPS URL of the MCP server.

          authorization: Raw Authorization header value (for example, 'Bearer <token>'). Datagrid stores
              it as a secret and links it to this server. If both authorization and
              authorization_secret_id are provided, authorization takes precedence.

          authorization_secret_id: Secret ID containing the full Authorization header value to use when calling
              this MCP server.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/organization/mcp-servers",
            body=await async_maybe_transform(
                {
                    "base_url": base_url,
                    "name": name,
                    "authorization": authorization,
                    "authorization_secret_id": authorization_secret_id,
                    "icon_url": icon_url,
                    "protocol_version": protocol_version,
                    "transport": transport,
                },
                mcp_server_create_params.McpServerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServer,
        )

    async def retrieve(
        self,
        server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServer:
        """
        Retrieve a registered MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not server_id:
            raise ValueError(f"Expected a non-empty value for `server_id` but received {server_id!r}")
        return await self._get(
            path_template("/organization/mcp-servers/{server_id}", server_id=server_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServer,
        )

    async def update(
        self,
        server_id: str,
        *,
        authorization: Optional[str] | Omit = omit,
        authorization_secret_id: Optional[str] | Omit = omit,
        base_url: str | Omit = omit,
        icon_url: Optional[str] | Omit = omit,
        name: str | Omit = omit,
        protocol_version: Optional[str] | Omit = omit,
        transport: Optional[Literal["http"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServer:
        """
        Update a registered MCP server.

        Args:
          authorization: Raw Authorization header value (for example, 'Bearer <token>'). Datagrid stores
              it as a secret and links it to this server. Set to null to clear. If both
              authorization and authorization_secret_id are provided, authorization takes
              precedence.

          authorization_secret_id: Secret ID containing the full Authorization header value to use when calling
              this MCP server. Set to null to clear.

          base_url: The HTTPS URL of the MCP server.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not server_id:
            raise ValueError(f"Expected a non-empty value for `server_id` but received {server_id!r}")
        return await self._patch(
            path_template("/organization/mcp-servers/{server_id}", server_id=server_id),
            body=await async_maybe_transform(
                {
                    "authorization": authorization,
                    "authorization_secret_id": authorization_secret_id,
                    "base_url": base_url,
                    "icon_url": icon_url,
                    "name": name,
                    "protocol_version": protocol_version,
                    "transport": transport,
                },
                mcp_server_update_params.McpServerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServer,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ListMcpServersResponse:
        """List registered MCP servers for the current teamspace."""
        return await self._get(
            "/organization/mcp-servers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ListMcpServersResponse,
        )

    async def delete(
        self,
        server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a registered MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not server_id:
            raise ValueError(f"Expected a non-empty value for `server_id` but received {server_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template("/organization/mcp-servers/{server_id}", server_id=server_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class McpServersResourceWithRawResponse:
    def __init__(self, mcp_servers: McpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = to_raw_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = to_raw_response_wrapper(
            mcp_servers.retrieve,
        )
        self.update = to_raw_response_wrapper(
            mcp_servers.update,
        )
        self.list = to_raw_response_wrapper(
            mcp_servers.list,
        )
        self.delete = to_raw_response_wrapper(
            mcp_servers.delete,
        )


class AsyncMcpServersResourceWithRawResponse:
    def __init__(self, mcp_servers: AsyncMcpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = async_to_raw_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            mcp_servers.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            mcp_servers.update,
        )
        self.list = async_to_raw_response_wrapper(
            mcp_servers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            mcp_servers.delete,
        )


class McpServersResourceWithStreamingResponse:
    def __init__(self, mcp_servers: McpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = to_streamed_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            mcp_servers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            mcp_servers.update,
        )
        self.list = to_streamed_response_wrapper(
            mcp_servers.list,
        )
        self.delete = to_streamed_response_wrapper(
            mcp_servers.delete,
        )


class AsyncMcpServersResourceWithStreamingResponse:
    def __init__(self, mcp_servers: AsyncMcpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = async_to_streamed_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            mcp_servers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            mcp_servers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            mcp_servers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            mcp_servers.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ConnectionProviderUpdateParams"]


class ConnectionProviderUpdateParams(TypedDict, total=False):
    client_id: str
    """The OAuth client ID to use for this connector."""

    client_secret: str
    """The OAuth client secret to use for this connector."""

    name: str
    """The name of the connection provider."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .service_account import ServiceAccount as ServiceAccount
from .service_account_credentials import ServiceAccountCredentials as ServiceAccountCredentials
from .service_account_list_params import ServiceAccountListParams as ServiceAccountListParams
from .service_account_create_params import ServiceAccountCreateParams as ServiceAccountCreateParams
from .service_account_list_response import ServiceAccountListResponse as ServiceAccountListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["ConversationUpdateParams"]


class ConversationUpdateParams(TypedDict, total=False):
    agent_ids: SequenceNotStr[str]
    """Replace the list of agents assigned to this conversation.

    Pass an empty array to clear all agent assignments.
    """

    name: str
    """Update the conversation name."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tables import (
    TablesResource,
    AsyncTablesResource,
    TablesResourceWithRawResponse,
    AsyncTablesResourceWithRawResponse,
    TablesResourceWithStreamingResponse,
    AsyncTablesResourceWithStreamingResponse,
)
from .knowledge import (
    KnowledgeResource,
    AsyncKnowledgeResource,
    KnowledgeResourceWithRawResponse,
    AsyncKnowledgeResourceWithRawResponse,
    KnowledgeResourceWithStreamingResponse,
    AsyncKnowledgeResourceWithStreamingResponse,
)

__all__ = [
    "TablesResource",
    "AsyncTablesResource",
    "TablesResourceWithRawResponse",
    "AsyncTablesResourceWithRawResponse",
    "TablesResourceWithStreamingResponse",
    "AsyncTablesResourceWithStreamingResponse",
    "KnowledgeResource",
    "AsyncKnowledgeResource",
    "KnowledgeResourceWithRawResponse",
    "AsyncKnowledgeResourceWithRawResponse",
    "KnowledgeResourceWithStreamingResponse",
    "AsyncKnowledgeResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ServiceAccountCredentials"]


class ServiceAccountCredentials(BaseModel):
    """The credentials for a service account."""

    object: Literal["service_account_credentials"]
    """The object type, which is always `service_account_credentials`."""

    private_key: str
    """The private key for the service account in JSON format."""

    type: Optional[Literal["gcp"]] = None
    """The type of service account credentials, currently only `gcp` is supported."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursorIDPage, AsyncCursorIDPage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.organization.teamspaces import user_list_params, user_update_params
from ....types.organization.teamspaces.teamspace_user import TeamspaceUser

__all__ = ["UsersResource", "AsyncUsersResource"]


class UsersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return UsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return UsersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        user_id: str,
        *,
        teamspace_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TeamspaceUser:
        """
        Retrieve details of a specific user in the teamspace.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not teamspace_id:
            raise ValueError(f"Expected a non-empty value for `teamspace_id` but received {teamspace_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get(
            path_template(
                "/organization/teamspaces/{teamspace_id}/users/{user_id}", teamspace_id=teamspace_id, user_id=user_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TeamspaceUser,
        )

    def update(
        self,
        user_id: str,
        *,
        teamspace_id: str,
        role: Literal["admin", "member", "agents-only", "agent-specific"],
        agent_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TeamspaceUser:
        """
        Update user permissions in the teamspace.

        Args:
          agent_ids: The agent IDs that the user has access to, if the role is agent-specific.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not teamspace_id:
            raise ValueError(f"Expected a non-empty value for `teamspace_id` but received {teamspace_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._patch(
            path_template(
                "/organization/teamspaces/{teamspace_id}/users/{user_id}", teamspace_id=teamspace_id, user_id=user_id
            ),
            body=maybe_transform(
                {
                    "role": role,
                    "agent_ids": agent_ids,
                },
                user_update_params.UserUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TeamspaceUser,
        )

    def list(
        self,
        teamspace_id: str,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        limit: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorIDPage[TeamspaceUser]:
        """
        Retrieve a list of users in the specified teamspace.

        Args:
          after: A cursor to use in pagination. `after` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              ending with `obj_foo`, your subsequent call can include `after=obj_foo` to fetch
              the next page of the list.

          before: A cursor to use in pagination. `before` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              starting with `obj_bar`, your subsequent call can include `before=obj_bar` to
              fetch the previous page of the list.

          limit: The limit on the number of objects to return, ranging between 1 and 100.

          search: Search term to filter users by name or email

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not teamspace_id:
            raise ValueError(f"Expected a non-empty value for `teamspace_id` but received {teamspace_id!r}")
        return self._get_api_list(
            path_template("/organization/teamspaces/{teamspace_id}/users", teamspace_id=teamspace_id),
            page=SyncCursorIDPage[TeamspaceUser],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "search": search,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            model=TeamspaceUser,
        )

    def delete(
        self,
        user_id: str,
        *,
        teamspace_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Revoke a user's permissions from the teamspace.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not teamspace_id:
            raise ValueError(f"Expected a non-empty value for `teamspace_id` but received {teamspace_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template(
                "/organization/teamspaces/{teamspace_id}/users/{user_id}", teamspace_id=teamspace_id, user_id=user_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncUsersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return AsyncUsersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        user_id: str,
        *,
        teamspace_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TeamspaceUser:
        """
        Retrieve details of a specific user in the teamspace.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not teamspace_id:
            raise ValueError(f"Expected a non-empty value for `teamspace_id` but received {teamspace_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._get(
            path_template(
                "/organization/teamspaces/{teamspace_id}/users/{user_id}", teamspace_id=teamspace_id, user_id=user_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TeamspaceUser,
        )

    async def update(
        self,
        user_id: str,
        *,
        teamspace_id: str,
        role: Literal["admin", "member", "agents-only", "agent-specific"],
        agent_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TeamspaceUser:
        """
        Update user permissions in the teamspace.

        Args:
          agent_ids: The agent IDs that the user has access to, if the role is agent-specific.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not teamspace_id:
            raise ValueError(f"Expected a non-empty value for `teamspace_id` but received {teamspace_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._patch(
            path_template(
                "/organization/teamspaces/{teamspace_id}/users/{user_id}", teamspace_id=teamspace_id, user_id=user_id
            ),
            body=await async_maybe_transform(
                {
                    "role": role,
                    "agent_ids": agent_ids,
                },
                user_update_params.UserUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TeamspaceUser,
        )

    def list(
        self,
        teamspace_id: str,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        limit: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TeamspaceUser, AsyncCursorIDPage[TeamspaceUser]]:
        """
        Retrieve a list of users in the specified teamspace.

        Args:
          after: A cursor to use in pagination. `after` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              ending with `obj_foo`, your subsequent call can include `after=obj_foo` to fetch
              the next page of the list.

          before: A cursor to use in pagination. `before` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              starting with `obj_bar`, your subsequent call can include `before=obj_bar` to
              fetch the previous page of the list.

          limit: The limit on the number of objects to return, ranging between 1 and 100.

          search: Search term to filter users by name or email

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not teamspace_id:
            raise ValueError(f"Expected a non-empty value for `teamspace_id` but received {teamspace_id!r}")
        return self._get_api_list(
            path_template("/organization/teamspaces/{teamspace_id}/users", teamspace_id=teamspace_id),
            page=AsyncCursorIDPage[TeamspaceUser],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "search": search,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            model=TeamspaceUser,
        )

    async def delete(
        self,
        user_id: str,
        *,
        teamspace_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Revoke a user's permissions from the teamspace.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not teamspace_id:
            raise ValueError(f"Expected a non-empty value for `teamspace_id` but received {teamspace_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template(
                "/organization/teamspaces/{teamspace_id}/users/{user_id}", teamspace_id=teamspace_id, user_id=user_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class UsersResourceWithRawResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.retrieve = to_raw_response_wrapper(
            users.retrieve,
        )
        self.update = to_raw_response_wrapper(
            users.update,
        )
        self.list = to_raw_response_wrapper(
            users.list,
        )
        self.delete = to_raw_response_wrapper(
            users.delete,
        )


class AsyncUsersResourceWithRawResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.retrieve = async_to_raw_response_wrapper(
            users.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            users.update,
        )
        self.list = async_to_raw_response_wrapper(
            users.list,
        )
        self.delete = async_to_raw_response_wrapper(
            users.delete,
        )


class UsersResourceWithStreamingResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.retrieve = to_streamed_response_wrapper(
            users.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            users.update,
        )
        self.list = to_streamed_response_wrapper(
            users.list,
        )
        self.delete = to_streamed_response_wrapper(
            users.delete,
        )


class AsyncUsersResourceWithStreamingResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.retrieve = async_to_streamed_response_wrapper(
            users.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            users.update,
        )
        self.list = async_to_streamed_response_wrapper(
            users.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            users.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from ..._models import BaseModel
from .service_account import ServiceAccount

__all__ = ["ServiceAccountListResponse"]


class ServiceAccountListResponse(BaseModel):
    data: List[ServiceAccount]
    """An array containing the service accounts."""

    object: Literal["list"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["CreditsReport"]


class CreditsReport(BaseModel):
    consumed: float
    """The number of credits consumed in the current billing period."""

    remaining: float
    """The number of unused credits remaining for the current billing period."""

    total: float
    """The initial total number of credits for the current billing period."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Connector", "OAuthAppSettings"]


class OAuthAppSettings(BaseModel):
    """Connector's OAuth app settings requirements."""

    redirect_uri: str
    """The OAuth redirect URI that must be configured in your OAuth app settings"""

    scopes: Optional[List[str]] = None
    """The OAuth scopes that must be granted when configuring your OAuth app."""


class Connector(BaseModel):
    """
    The `connector` object represents an available connector that can be used to connect to a third-party service.
    """

    id: str
    """The unique identifier for the connector."""

    name: str
    """The display name of the connector."""

    object: Literal["connector"]
    """The object type, which is always `connector`."""

    oauth_app_settings: Optional[OAuthAppSettings] = None
    """Connector's OAuth app settings requirements."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Conversation"]


class Conversation(BaseModel):
    """The `conversation` object represents a conversation with an AI agent."""

    id: str
    """The conversation identifier, which can be referenced in the API endpoints."""

    created_at: datetime
    """The ISO string for when the conversation was created."""

    object: Literal["conversation"]
    """The object type, which is always `conversation`."""

    updated_at: datetime
    """The ISO string for when the conversation was last updated."""

    agent_ids: Optional[List[str]] = None
    """Array of agent IDs currently assigned to this conversation."""

    name: Optional[str] = None
    """The name of the conversation."""

    participated_agent_ids: Optional[List[str]] = None
    """Array of agent IDs that have previously responded in this conversation.

    This list only grows and is never cleared when agents are reassigned.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["IFrameEventType"]

IFrameEventType: TypeAlias = Literal[
    "datagrid-api/error",
    "datagrid-api/connection-created",
    "datagrid-api/connection-updated",
    "datagrid-api/content-loaded",
    "datagrid-api/resize",
    "datagrid-api/knowledge-created",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ServiceAccountCreateParams"]


class ServiceAccountCreateParams(TypedDict, total=False):
    name: Required[str]
    """The name of the service account.

    Your organization's domain will be automatically prepended to the service
    account name. The name must only include letters (a-z, A-Z), numbers (0-9), and
    hyphens (-), and must be between 6 and 30 characters long.
    """

    type: Required[Literal["gcp"]]
    """The type of service account, currently only `gcp` is supported."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .tool_name import ToolName

__all__ = ["Tool"]


class Tool(BaseModel):
    name: ToolName
    """The unique identifier for a tool."""

    connection_id: Optional[str] = None
    """The ID of the connection to use for the tool."""
